/*
    ContactXfer - Transfer Contacts via Wireless Network

    Copyright (C) 2008  Nigel D. Stepp

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Nigel Stepp <stepp@atistar.net>
    http://www.atistar.net/~stepp/contactxfer/

    $Id: BrowserViewController.m 282 2008-09-02 22:56:58Z stepp $
*/

#import "BrowserViewController.h"
#import "ContactXferViewController.h"

@implementation BrowserViewController

@synthesize browserView;
@synthesize services;
@synthesize resolvingServices;

- (id)initWithStyle:(UITableViewStyle)style {
	if (self = [super initWithStyle:style]) {
	}
	return self;
}

- (void)show {
	browserView.hidden = NO;
}
- (void)hide {
	browserView.hidden = YES;
}

- (void)setStatusDelegate:(id)delegate {
	statusDelegate = delegate;
}

- (void)handleError:(NSNumber *)error {
    NSLog(@"An error occurred. Error code = %d", [error intValue]);
    // Handle error here	
}



- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	NSInteger rows = 0;
	
	if( services ) {
		rows = [services count];
	}
	
	return rows;
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	
	static NSString *MyIdentifier = @"MyIdentifier";
	
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:MyIdentifier];
	if (cell == nil) {
		cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:MyIdentifier] autorelease];
	}
	
	NSNetService *service = [services objectAtIndex:indexPath.row];
	cell.text = [service name];

	return cell;
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	NSNetService *service = [services objectAtIndex:indexPath.row];
	
	[tableView deselectRowAtIndexPath:indexPath animated:NO];

	// Notify the main controller of this selection
	ContactXferViewController *parent = (ContactXferViewController *)statusDelegate;
	[parent didFindService:service];

	[services removeAllObjects];
}

/*
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
	
	if (editingStyle == UITableViewCellEditingStyleDelete) {
	}
	if (editingStyle == UITableViewCellEditingStyleInsert) {
	}
}
*/
/*
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
	return YES;
}
*/
/*
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/
/*
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
	return YES;
}
*/

- (BOOL)tableView:(UITableView *)tableView shouldIndentWhileEditingRowAtIndexPath:(NSIndexPath *)indexPath{
	return NO;
}

- (NSIndexPath *)tableView:(UITableView *)tableView targetIndexPathForMoveFromRowAtIndexPath:(NSIndexPath *)sourceIndexPath toProposedIndexPath:(NSIndexPath *)proposedDestinationIndexPath {
	return proposedDestinationIndexPath;
}


- (void)dealloc {
	[services release];
	[super dealloc];
}


- (void)viewDidLoad {
	[super viewDidLoad];
}


- (void)viewWillAppear:(BOOL)animated {
	[super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated {
	[super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated {
}

- (void)viewDidDisappear:(BOOL)animated {
}

- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning];
}

- (void)updateUI {
	[browserView reloadData];
	dirty = NO;
}

// Sent when browsing begins
- (void)netServiceBrowserWillSearch:(NSNetServiceBrowser *)browser {
    searching = YES;
    [self updateUI];
}


// Sent when browsing stops
- (void)netServiceBrowserDidStopSearch:(NSNetServiceBrowser *)browser {
    searching = NO;
	[services removeAllObjects];
    [self updateUI];
}


// Sent if browsing fails
- (void)netServiceBrowser:(NSNetServiceBrowser *)browser
			 didNotSearch:(NSDictionary *)errorDict {
    searching = NO;
    [self handleError:[errorDict objectForKey:NSNetServicesErrorCode]];
}


// Sent when a service appears
- (void)netServiceBrowser:(NSNetServiceBrowser *)browser
		   didFindService:(NSNetService *)aNetService
			   moreComing:(BOOL)moreComing {

	if( !resolvingServices ) {
		resolvingServices = [[NSMutableArray alloc] init];
	}

	[resolvingServices addObject:aNetService];

	[aNetService setDelegate:self];
	[aNetService resolveWithTimeout:5.0];

	if(!moreComing) {
		//[self updateUI];
		dirty = YES;
	}
}


// Sent when a service disappears
- (void)netServiceBrowser:(NSNetServiceBrowser *)browser
		 didRemoveService:(NSNetService *)aNetService
			   moreComing:(BOOL)moreComing {
	
	[resolvingServices removeObject:aNetService];
    [services removeObject:aNetService];
	
    if(!moreComing) {
        [self updateUI];
    }
}

// Sent when addresses are resolved
- (void)netServiceDidResolveAddress:(NSNetService *)netService {
    // Make sure [netService addresses] contains the
    // necessary connection information
    if ([self addressesComplete:[netService addresses]
				 forServiceType:[netService type]]) {

		if( !services ) {
			services = [[NSMutableArray alloc] init];
		}

        [services addObject:netService];
		[resolvingServices removeObject:netService];
		
		if( dirty ) {
			[self updateUI];
		}
    }
}


// Sent if resolution fails
- (void)netService:(NSNetService *)netService
	 didNotResolve:(NSDictionary *)errorDict {
    [self handleError:[errorDict objectForKey:NSNetServicesErrorCode]];
    [resolvingServices removeObject:netService];	
	[self updateUI];
}


// Verifies [netService addresses]
- (BOOL)addressesComplete:(NSArray *)addresses
		   forServiceType:(NSString *)serviceType {
    // Perform appropriate logic to ensure that [netService addresses]
    // contains the appropriate information to connect to the service
 
	if( [addresses count] <= 0 ) {
		return NO;
	}
	
	return YES;	
}

@end

