/*
    ContactXfer - Transfer Contacts via Wireless Network

    Copyright (C) 2008  Nigel D. Stepp

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Nigel Stepp <stepp@atistar.net>
    http://www.atistar.net/~stepp/contactxfer/

    $Id: ContactXferViewController.m 286 2008-09-05 06:30:02Z stepp $
*/

#import "ContactXferViewController.h"
#import "BrowserViewController.h"
#import <AddressBook/AddressBook.h>
#import <AddressBookUI/AddressBookUI.h>
#import <AddressBook/ABPerson.h>
#import <SystemConfiguration/SCNetworkReachability.h>
#import "NetworkConnection.h"
#import "PersonSerializer.h"

@implementation ContactXferViewController

@synthesize sendButton;
@synthesize receiveButton;
@synthesize cancelButton;
@synthesize infoButton;
@synthesize indicator;
@synthesize statusMessage;
@synthesize addrOutImg;
@synthesize addrInImg;
@synthesize browserController;
@synthesize serviceBrowser;
@synthesize tcpServer;
@synthesize contactData;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
		// Initialization code
	}
	return self;
}

/*
 Implement loadView if you want to create a view hierarchy programmatically
- (void)loadView {
}
 */

/*
 If you need to do additional setup after loading the view, override viewDidLoad.
- (void)viewDidLoad {
}
 */


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
	// Release anything that's not essential, such as cached data
}


- (void)dealloc {
	[sendButton dealloc];
	[receiveButton dealloc];
	[cancelButton dealloc];
	[infoButton dealloc];
	[indicator dealloc];
	[statusMessage dealloc];	
	
	[super dealloc];
}


// UI Actions
-(IBAction)startSend:(id)sender {
	if( ![self checkReachability] ) {
		[self warnNoNetwork];
		return;
	}

	statusMessage.text = @"Sending...";
	[indicator startAnimating];

	ABPeoplePickerNavigationController *picker =
	
	[[ABPeoplePickerNavigationController alloc] init];
	
    picker.peoplePickerDelegate = self;
	
    [self presentModalViewController:picker animated:YES];
	
    [picker release];
	
}

-(IBAction)startReceive:(id)sender {

	if( ![self checkReachability] ) {
		[self warnNoNetwork];
		return;
	}

	addrInImg.hidden = NO;
	statusMessage.text = @"Receiving...";
	[indicator startAnimating];
	
	NetworkConnection *connection = [[NetworkConnection alloc] init];
	if( ![connection startListenWithDelegate:self] ) {
		statusMessage.text = @"Receive failed :(";
	}
	tcpServer = connection.tcpServer;
}

-(IBAction)userCancel:(id)sender{
	statusMessage.text = @"Canceling...";
	[indicator stopAnimating];
	
	[serviceBrowser stop];
	[tcpServer stop];

	[browserController hide];
	[contactData release];
	addrInImg.hidden = YES;
	addrOutImg.hidden = YES;
	statusMessage.text = @"";
}

-(IBAction)showInfo:(id)sender {
	NSString *titleString = [NSString 
							 stringWithFormat:@"ContactXfer %@", version];
	NSString *aboutMessage = [NSString 
							 stringWithFormat:@"\
Copyright (C) 2008 Nigel Stepp\n\
This program is free software, see \
website for license details.\n\n\
Thank you for using ContactXfer. \
For more information, please see \
http://atistar.net/~stepp/contactxfer/\n\n\
Send bug reports, feature requests, \
or any other feedback, to:\n\
Nigel Stepp <stepp@atistar.net>\
"];

	UIAlertView *alertView = [[UIAlertView alloc]
							  initWithTitle:titleString
							  message:aboutMessage
							  delegate:nil
							  cancelButtonTitle:@"Ok"
							  otherButtonTitles:nil];
	
	[alertView show];
}

-(void)warnNoNetwork {
	NSString *titleString = @"Network Availability";
	NSString *warnMessage = [NSString 
							 stringWithFormat:@"\
ContactXfer communicates over a \
network, but can't find one.\n\n\
Unfortunately, iPhones are not \
capable of hosting their own \
wireless networks. An existing \
network is required (Ad-Hoc is ok).\n\n\
If only Bluetooth worked... *sigh*\
"];

	UIAlertView *alertView = [[UIAlertView alloc]
							  initWithTitle:titleString
							  message:warnMessage
							  delegate:nil
							  cancelButtonTitle:@"Bummer"
							  otherButtonTitles:nil];
	
	[alertView show];
}

// Determine network reachability. We need a WiFi
// connection, not carrier network.
-(bool)checkReachability {
	SCNetworkReachabilityRef defaultRouteReachability = nil;

	struct sockaddr_in zeroAddress;
	bzero(&zeroAddress, sizeof(zeroAddress));
	zeroAddress.sin_len = sizeof(zeroAddress);
	zeroAddress.sin_family = AF_INET;

	defaultRouteReachability = SCNetworkReachabilityCreateWithAddress(NULL, (struct sockaddr *)&zeroAddress);

	SCNetworkReachabilityFlags flags;
	BOOL gotFlags = SCNetworkReachabilityGetFlags(defaultRouteReachability, &flags);

	if (!gotFlags) {
		return NO;
	}

	// WiFi if reachable but not on the carrier network
	if( flags & kSCNetworkReachabilityFlagsReachable
			&& !(flags & kSCNetworkReachabilityFlagsIsWWAN) ) {
		return YES;
	}

	return NO;
}



// Net Connection delegate methods

-(void)didGetData:(NSData *)data {
	ABRecordRef person;
	
	// Deserialize data
	PersonSerializer *serializer = [[PersonSerializer alloc] init];
	person = [serializer deserializeData:data];
	[serializer release];
	
	
	addrInImg.hidden = YES;
	[indicator stopAnimating];
	
	statusMessage.text = @"Received data";
	
	// throw up new contact code
	ABNewPersonViewController *newPersonView = [[ABNewPersonViewController alloc] init];
	newPersonView.newPersonViewDelegate = self;
	newPersonView.displayedPerson = person;
	
	UINavigationController *newPersonNavView = [[UINavigationController alloc] initWithRootViewController:newPersonView];
    
	[self presentModalViewController:newPersonNavView animated:YES];

	[newPersonView release];
	[newPersonNavView release];
}

-(void)didNotGetData:(NSString *)error {
	addrInImg.hidden = YES;
	[indicator stopAnimating];
	
	statusMessage.text = @"Receive failed :(";
}

// Address Book Delegate methods

- (void)newPersonViewController:(ABNewPersonViewController *)newPersonViewController didCompleteWithNewPerson:(ABRecordRef)person {
	statusMessage.text = @"Added contact";
	[self dismissModalViewControllerAnimated:YES];
}

- (void)peoplePickerNavigationControllerDidCancel:(ABPeoplePickerNavigationController *)peoplePicker {
    [self dismissModalViewControllerAnimated:YES];
	[self userCancel:nil];
}

- (BOOL)peoplePickerNavigationController:(ABPeoplePickerNavigationController *)peoplePicker
	  shouldContinueAfterSelectingPerson:(ABRecordRef)person {
	
	PersonSerializer *serializer = [[PersonSerializer alloc] init];
	contactData = [[serializer serializePerson:person] retain];
	[serializer release];
	
	// Now we have data to send
	[self haveDataToSend];
	
	[self dismissModalViewControllerAnimated:YES];

    return NO;
}

- (void)haveDataToSend {
	
	// We need to pick a receiver
	// Start a service browser and show the browser
	// table view.

	if( !serviceBrowser ) {
		serviceBrowser = [[NSNetServiceBrowser alloc] init];
		[serviceBrowser setDelegate:browserController];
	}
	[serviceBrowser searchForServicesOfType:@"_cxfer._tcp" inDomain:@"local."];

	[browserController show];
	addrOutImg.hidden = NO;
	addrOutImg.alpha = 0.25;
	statusMessage.text = @"Listening for receivers...";
}

- (void)didFindService:(NSNetService *)service {

	[browserController hide];
	addrOutImg.alpha = 1.0;
	statusMessage.text = [NSString stringWithFormat:@"Connecting to %@...", [service name]];
	[serviceBrowser stop];	

	if( !contactData || [contactData length] <= 0 ) {
		NSLog(@"Found a receiver, but I now have no data.");
		statusMessage.text = @"Sending failed, no data.";
		return;
	}
	
	// This could block...
	[NSThread detachNewThreadSelector:@selector(sendDataWithService:) toTarget:self withObject:service];
}

- (void)sendDataWithService:(NSNetService *)service {
	// resolve service, make connection, send data.
	// this should be done in a networking class
	NetworkConnection *connection = [[NetworkConnection alloc] init];
	if( [connection sendWithService:service data:contactData] ) {
		statusMessage.text = @"Contact sent sucessfully";
	} else {
		statusMessage.text = @"Sending failed :(";
	}
	
	addrOutImg.hidden = YES;
	
	[connection release];
	[contactData release];
	contactData = nil;
	
	[indicator stopAnimating];
}

- (BOOL)peoplePickerNavigationController:(ABPeoplePickerNavigationController *)peoplePicker
      shouldContinueAfterSelectingPerson:(ABRecordRef)person
                                property:(ABPropertyID)property
                              identifier:(ABMultiValueIdentifier)identifier{
    return NO;
}


@end
