/*
    ContactXfer - Transfer Contacts via Wireless Network

    Copyright (C) 2008  Nigel D. Stepp

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Nigel Stepp <stepp@atistar.net>
    http://www.atistar.net/~stepp/contactxfer/

    $Id: NetworkConnection.m 282 2008-09-02 22:56:58Z stepp $
*/

#import "NetworkConnection.h"


@implementation NetworkConnection

@synthesize tcpServer;
@synthesize delegateObject;
@synthesize inStream;

-(id)init {
	self = [super init];
	address = nil;
	tcpServer = nil;

	return self;
}

-(void)dealloc {
	[tcpServer release];
	[super dealloc];
}

-(BOOL) sendWithService:(NSNetService *)service data:(NSData *)sendData {

	int remoteSocket;
	int connectResult = -1;
	NSFileHandle *remoteFile;
	
	// Make Connection and send data	
	if([[service addresses] count] > 0) {
		
		address = (struct sockaddr *)[[[service addresses] objectAtIndex:0] bytes];
		remoteSocket = socket(address->sa_family, SOCK_STREAM, 0);
		
		if(remoteSocket > 0) {
	
			remoteFile = [[NSFileHandle alloc]
						  initWithFileDescriptor:remoteSocket closeOnDealloc:YES];
			
			if(remoteFile) {
				
				connectResult = connect(remoteSocket, address, address->sa_len);
				
				if(connectResult == 0) {
					[remoteFile writeData:sendData];
				} else {
					NSLog(@"An error occurred connecting to the socket.");
				}
				
				[remoteFile closeFile];
				
			} else {
				NSLog(@"Could not create NSFileHandle object.");
			}
		} else {
			NSLog(@"Could not create socket.");
		}
	} else {
		NSLog(@"Could not find any addresses for the service.");
	}
	
	if( connectResult != 0 ) {
		return NO;
	}
	
	return YES;
}

-(BOOL) startListenWithDelegate:(id)delegate {
	delegateObject = delegate;

	// Create a new TCPServer
	if( !tcpServer ) {
		tcpServer = [[TCPServer alloc] init];
		tcpServer.delegate = self;
	}

	NSError *error;
	if( ![tcpServer start:&error] ) {
		NSLog(@"Error starting tcp server");
		return NO;
	}

	if( ![tcpServer enableBonjourWithDomain:@"local." applicationProtocol:@"_cxfer._tcp." name:@""] ) {
		[tcpServer stop];
		NSLog(@"Error starting Bonnjour service.");
		return NO;
	}
	
	return YES;
}

- (void) didAcceptConnectionForServer:(TCPServer*)server inputStream:(NSInputStream *)istr outputStream:(NSOutputStream *)ostr {

	// We got a connection, so receive the data
	inStream = istr;
	[inStream retain];
	inStream.delegate = self;
	[inStream scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
	[inStream open];
}
/*	
	len = [istr read:buffer maxLength:BUFFER_SIZE];
	if( len <= 0 ) {
		NSLog(@"Error reading from input stream.");
		// alloc an NSError object here
		if( delegateObject ) {
			[delegateObject didNotGetData:@"Error reading from input stream"];
		}
		return;
	}

	data = [[NSData dataWithBytes:buffer length:len] retain];

	[tcpServer stop];

	if( delegateObject ) {
		[delegateObject didGetData:data];
	}
}
*/
// Stream Delegate Methods
- (void) stream:(NSStream*)stream handleEvent:(NSStreamEvent)eventCode {
	uint8_t buffer[BUFFER_SIZE];
	NSInteger len;
	NSData *data;

	switch(eventCode) {
		case NSStreamEventOpenCompleted:
		{
			// release server??
			break;
		}
		case NSStreamEventHasBytesAvailable:
		{
			if (stream == inStream) {
				len = [inStream read:buffer maxLength:BUFFER_SIZE];
				if( len <= 0 ) {
					NSLog(@"Error reading from input stream.");
					// alloc an NSError object here
					if( delegateObject ) {
						[delegateObject didNotGetData:@"Error reading from input stream"];
					}
					return;
				}
				
				data = [[NSData dataWithBytes:buffer length:len] retain];
				
				[tcpServer stop];
				[inStream close];
				[inStream removeFromRunLoop:[NSRunLoop currentRunLoop]
								  forMode:NSDefaultRunLoopMode];
				[inStream release];
				inStream = nil;
				
 				
				if( delegateObject ) {
					[delegateObject didGetData:data];
				}
			}
			break;
		}
		case NSStreamEventEndEncountered:
		{
			[stream close];			
            [stream removeFromRunLoop:[NSRunLoop currentRunLoop]
							  forMode:NSDefaultRunLoopMode];
            [stream release];
			
            stream = nil; // stream is ivar, so reinit it
			
			break;
		}
	}
}

@end

